/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef BITENUMS_H
#define BITENUMS_H

namespace bit
{

  /**
  * Convenience enumeration associating constant names with the most common unit sizes.
   */
  typedef enum UNITS {
    BITS = 1,
    OCTETS = 8
  } UNITS;

  /**
   * List of field/record container types
   */
  typedef enum Container {
    FIELD,
    FIELD_VECTOR,
    FIELD_ELEMENT,
    RECORD,
    RECORD_VECTOR,
    RECORD_ELEMENT,
  } Container;

  /**
   * Allowable field type assignments
   */
  typedef enum Type {
    TYPE_NONE,
    TYPE_INTEGER,
    TYPE_FLOATING,
    TYPE_ASCII,
    TYPE_UTF8,
    TYPE_BCD
  } Type;

  /**
   * Allowable byte orderings
   */
  typedef enum ByteOrder {
    BYTEORDER_HOST,
    BYTEORDER_NETWORK,
    BYTEORDER_BIG_ENDIAN,
    BYTEORDER_LITTLE_ENDIAN,
  } ByteOrder;

  /**
   * Allowable integer sign representations
   */
  typedef enum Sign {
    SIGN_UNSIGNED,
    SIGN_TWOS_COMPLEMENT,
    SIGN_ONES_COMPLEMENT,
    SIGN_BIT,
  } Sign;

  /**
   * Used to define a packing/unpacking location with respect to a large memory chunk.
   */
  typedef enum Location {
    START,  /**< packs/unpacks data at the start of the memory location, possibly leaving the end untouched */
    END,    /**< packs/unpacks data at the end of the memory location, possibly leaving the start untouched */
  } Location;

  typedef enum IntegerType {
    UINT64,
    INT64,
    UINT32,
    INT32,
    UINT16,
    INT16,
    UINT8,
    INT8,
  } IntegerType;


}

#endif
