/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "fieldbase.h"

#include "record.h"

#define WHOLE_UNITS(__offset, __units) \
  ( ((__offset) > 0.0) ? ( (__offset)/__units+(((__offset)%__units)?1:0) ) : ( (__offset)/__units+(((__offset)%__units)?(-1):0)) )

namespace bit
  {

  FieldBase::FieldBase(): m_parent(NULL)
  {}

  FieldBase::~FieldBase()
  {}

  size_t FieldBase::length(size_t units) const
    {
      return ( WHOLE_UNITS( this->length() * this->length_units(), units ) );
    }

  int FieldBase::offset(size_t units) const
  {
    return ( WHOLE_UNITS( this->offset() * this->offset_units(), units ) );
  }

  int FieldBase::start(size_t units) const
  {
    if (m_parent != NULL)
      return ( WHOLE_UNITS(m_parent->start(BITS) + this->offset(BITS), units));
    else
      return WHOLE_UNITS(this->offset(BITS), units);
  }

  std::string FieldBase::name(int depth) const {
    if (m_parent == NULL || depth == 0)
      return name();
    return m_parent->name(depth-1) + std::string(".") + this->name();

  }

  size_t FieldBase::depth() const {
    if (m_parent == NULL)
      return 0;
    else
      return m_parent->depth() + 1;
  }

  const FieldType& FieldBase::type() {
    return m_type;
  }

  void FieldBase::set_type(const FieldType& type) {
    m_type = type;
  }

  bool FieldBase::operator<(const FieldBase& other) const
    {
      if ( this->start(BITS) < other.start(BITS) )
        return true;
      if ( this->start(BITS) == other.start(BITS) )
        return this->length(BITS) < other.length(BITS);
      return false;
    }

  bool FieldBase::operator>(const FieldBase& other) const
    {
      if ( this->start(BITS)+this->length(BITS) > other.start(BITS)+other.length(BITS) )
        return true;
      if ( this->start(BITS)+this->length(BITS) == other.start(BITS)+other.length(BITS) )
        return this->start(BITS) > other.start(BITS);
      return false;
    }


  //     class iterator: public std::iterator<std::bidirectional_iterator_tag, FieldBase> {

  FieldBase::iterator::iterator(FieldBase* container, FieldBase::pointer object):
      m_container(container), m_object(object)
{ }

  FieldBase::iterator::~iterator()
  { }

  FieldBase::iterator& FieldBase::iterator::operator=(const iterator& other) {
    m_container = other.m_container;
    m_object = other.m_object;
    return *this;
  }

    bool FieldBase::iterator::operator==(const iterator& other) const {
      return (m_container == other.m_container && m_object == other.m_object);
    }

    bool FieldBase::iterator::operator!=(const iterator& other) const {
      return (m_container != other.m_container || m_object != other.m_object);
    }

  FieldBase::iterator& FieldBase::iterator::operator++() throw (exception::invalid_iterator)
  {
    if (! m_object)
      throw exception::invalid_iterator();
    m_object = m_container->next_field( m_object );
    return *this;
  }

  FieldBase::iterator FieldBase::iterator::operator++(int) throw (exception::invalid_iterator)
  {
    if (! m_object)
      throw exception::invalid_iterator();
    m_object = m_container->next_field( m_object );
    return *this;
  }

  FieldBase::iterator& FieldBase::iterator::operator--() throw (exception::invalid_iterator)
  {
    if (! m_object)
      throw exception::invalid_iterator();
    m_object = m_container->previous_field( m_object );
    return *this;
  }

  FieldBase::iterator FieldBase::iterator::operator--(int) throw (exception::invalid_iterator)
  {
    if (! m_object)
      throw exception::invalid_iterator();
    m_object = m_container->previous_field( m_object );
    return *this;
  }

  FieldBase& FieldBase::iterator::operator*() throw (exception::invalid_iterator)
  {
    if (m_object)
      return *m_object;
    else
      throw ( exception::invalid_iterator() );
  }

  FieldBase* FieldBase::iterator::operator->() throw (exception::invalid_iterator)
  {
    if (m_object)
      return m_object.get();
    else
      throw ( exception::invalid_iterator() );
  }

  FieldBase::pointer FieldBase::iterator::pointer()
  {
    return m_object;
  }

  FieldBase::iterator FieldBase::begin()
  {
    return this->end();
  }

  FieldBase::iterator FieldBase::end()
  {
    return iterator(this, FieldBase::pointer());
  }

  FieldBase& FieldBase::operator[](std::string s) throw (exception::invalid_container_op, std::out_of_range)
  {
    FieldBase::pointer f = this->field(s);
    if (!f)
      throw ( exception::invalid_container_op() );
    return *f;
  }

  FieldBase& FieldBase::operator[](size_t i) throw (exception::invalid_container_op, std::out_of_range)
  {
    FieldBase::pointer f = this->field(i);
    if (!f)
    throw ( exception::invalid_container_op() );
    return *f;
  }

  FieldBase::pointer FieldBase::field(std::string s) {
    return FieldBase::pointer();
  }

  FieldBase::pointer FieldBase::field(size_t i) {
    return FieldBase::pointer();
  }

  FieldBase::pointer FieldBase::previous_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {
    throw ( exception::invalid_container_op() );
  }

  FieldBase::pointer FieldBase::next_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {
    throw ( exception::invalid_container_op() );
  }

  size_t FieldBase::fields() {
    return 0;
  }

}
